@@ -1,6 +1,6 @@
 package Memcached::Client::Protocol::Text;
 BEGIN {
-  $Memcached::Client::Protocol::Text::VERSION = '0.97';
+  $Memcached::Client::Protocol::Text::VERSION = '0.98';
 }
 # ABSTRACT: Implements original text-based memcached protocol
 
@@ -18,7 +18,7 @@ sub __cmd {
         my ($name) = @_;
         sub {
             my ($self, $handle, $cv, $key, $value, $flags, $expiration) = @_;
-            DEBUG "P: %s: %s - %s - %s", $name, $handle->{peername}, $key, $value;
+            # DEBUG "P: %s: %s - %s - %s", $name, $handle->{peername}, $key, $value;
             my $command = __cmd ($name, $key, $flags, $expiration, length $value) . __cmd ($value);
             $handle->push_write ($command);
             $handle->push_read (line => sub {
@@ -41,8 +41,9 @@ sub __cmd {
         my ($name) = @_;
         return sub {
             my ($self, $handle, $cv, $key, $delta, $initial) = @_;
-            DEBUG "P: %s: %s - %s - %s", $name, $handle->{peername}, $key, $delta;
+            # DEBUG "P: %s: %s - %s - %s", $name, $handle->{peername}, $key, $delta;
             my $command = __cmd ($name, $key, $delta);
+            # DEBUG "P [%s]: > %s", $handle->{peername}, $command;
             $handle->push_write ($command);
             $handle->push_read (line => sub {
                                     my ($handle, $line) = @_;
@@ -73,10 +74,10 @@ sub __cmd {
 
 sub __delete {
     my ($self, $handle, $cv, $key) = @_;
-    DEBUG "P: delete: %s - %s", $handle->{peername}, $key;
+    # DEBUG "P: delete: %s - %s", $handle->{peername}, $key;
     my $command = __cmd (delete => $key);
     $handle->push_write ($command);
-    #DEBUG "P [%s]: > %s", $handle->{peername}, $command;
+    # DEBUG "P [%s]: > %s", $handle->{peername}, $command;
     $handle->push_read (line => sub {
                             my ($handle, $line) = @_;
                             # DEBUG "P [%s]: < %s", $handle->{peername}, $line;
@@ -88,7 +89,7 @@ sub __flush_all {
     my ($self, $handle, $cv, $delay) = @_;
     my $command = $delay ? __cmd (flush_all => $delay) : __cmd ("flush_all");
     $handle->push_write ($command);
-    DEBUG "P: flush_all: %s", $handle->{peername};
+    # DEBUG "P: flush_all: %s", $handle->{peername};
     # DEBUG "P [%s]: > %s", $handle->{peername}, $command;
     $handle->push_read (line => sub {
                             my ($handle, $line) = @_;
@@ -100,7 +101,7 @@ sub __flush_all {
 sub __get {
     my ($self, $handle, $cv, @keys) = @_;
     for my $key (@keys) {
-        DEBUG "P: get: %s - %s", $handle->{peername}, $key;
+        # DEBUG "P: get: %s - %s", $handle->{peername}, $key;
     }
     my $command = __cmd (get => @keys);
     $handle->push_write ($command);
@@ -136,7 +137,7 @@ sub __stats {
     my ($self, $handle, $cv, $name) = @_;
     my $command = $name ? __cmd (stats => $name) : __cmd ("stats");
     $handle->push_write ($command);
-    DEBUG "P: stats: %s", $handle->{peername};
+    # DEBUG "P: stats: %s", $handle->{peername};
     # DEBUG "P [%s]: > %s", $handle->{peername}, $command;
     my ($result);
     my $code; $code = sub {
@@ -159,7 +160,7 @@ sub __version {
     my ($self, $handle, $cv) = @_;
     my $command = __cmd ("version");
     $handle->push_write ($command);
-    DEBUG "P: version: %s", $handle->{peername};
+    # DEBUG "P: version: %s", $handle->{peername};
     # DEBUG "P [%s]: > %s", $handle->{peername}, $command;
     $handle->push_read (line => sub {
                             my ($handle, $line) = @_;
@@ -185,7 +186,7 @@ Memcached::Client::Protocol::Text - Implements original text-based memcached pro
 
 =head1 VERSION
 
-version 0.97
+version 0.98
 
 =head1 AUTHOR
 
